import BaseAPI from "./BaseAPI";
import { CMI } from "./cmi/scorm2004/cmi";
import { ResponseType } from "./constants/response_constants";
import { CMIArray } from "./cmi/common/array";
import { BaseCMI } from "./cmi/common/base_cmi";
import { CMIInteractionsObject } from "./cmi/scorm2004/interactions";
import { CMIObjectivesObject } from "./cmi/scorm2004/objectives";
import { ADL } from "./cmi/scorm2004/adl";
import { CommitObject, ResultObject, Settings } from "./types/api_types";
declare class Scorm2004Impl extends BaseAPI {
    private _version;
    private _globalObjectives;
    constructor(settings?: Settings);
    cmi: CMI;
    adl: ADL;
    Initialize: () => string;
    Terminate: () => string;
    GetValue: (CMIElement: string) => string;
    SetValue: (CMIElement: string, value: any) => string;
    Commit: () => string;
    GetLastError: () => string;
    GetErrorString: (CMIErrorCode: string | number) => string;
    GetDiagnostic: (CMIErrorCode: string | number) => string;
    reset(settings?: Settings): void;
    get version(): string;
    get globalObjectives(): CMIObjectivesObject[];
    lmsInitialize(): string;
    lmsFinish(): string;
    internalFinish(): Promise<string>;
    lmsGetValue(CMIElement: string): string;
    lmsSetValue(CMIElement: string, value: any): string;
    lmsCommit(): string;
    lmsGetLastError(): string;
    lmsGetErrorString(CMIErrorCode: string | number): string;
    lmsGetDiagnostic(CMIErrorCode: string | number): string;
    setCMIValue(CMIElement: string, value: any): string;
    getChildElement(CMIElement: string, value: any, foundFirstIndex: boolean): BaseCMI | null;
    private createCorrectResponsesObject;
    checkValidResponseType(response_type: ResponseType, value: any, interaction_type: string): void;
    checkDuplicateChoiceResponse(interaction: CMIInteractionsObject, value: any): void;
    validateCorrectResponse(CMIElement: string, value: any): void;
    getCMIValue(CMIElement: string): any;
    getLmsErrorMessageDetails(errorNumber: string | number, detail: boolean): string;
    checkDuplicatedPattern(correct_response: CMIArray, current_index: number, value: any): boolean;
    checkCorrectResponseValue(interaction_type: string, nodes: Array<any>, value: any): void;
    removeCorrectResponsePrefixes(node: string): any;
    replaceWithAnotherScormAPI(newAPI: Scorm2004Impl): void;
    renderCommitCMI(terminateCommit: boolean): object | Array<any>;
    renderCommitObject(terminateCommit: boolean): CommitObject;
    storeData(terminateCommit: boolean): Promise<ResultObject>;
}
export { Scorm2004Impl as Scorm2004API };
